title: Pure CSS Parallax Websites
url: http://keithclark.co.uk/articles/pure-css-parallax-websites/
hash_url: bdea3b9e4326f5709a02c72c720367db

<p>This article demonstrates how to use CSS transforms, perspective and some scaling trickery to create a pure CSS parallax scrolling website.</p>
<p>Parallax is almost always handled with JavaScript and, more often than not, it's implemented badly with the worst offenders listening for the <code>scroll</code> event and modifying the DOM directly in the handler, triggering needless reflows and paints. All this happens out of sync with the browsers rendering pipeline causing dropped frames and stuttering. It's not all bad though, <code>requestAnimationFrame</code> and deferring DOM updates can transform parallax websites - but what if you could remove the JavaScript dependency completely?</p>
<p>Deferring the parallax effect to CSS removes all these issues and allows the browser to leverage hardware acceleration resulting in almost everything being handled by the compositor. The result is consistent frame rates and perfectly smooth scrolling. You can also combine the effect with other CSS features such as <a href="http://www.w3.org/TR/css3-mediaqueries/">media queries</a> or <a href="http://www.w3.org/TR/css3-conditional/">supports</a> - responsive parallax anyone?</p>
<p><a class="button" href="http://keithclark.co.uk/articles/pure-css-parallax-websites/demo3/">Try the demo</a></p>
<h2>The theory</h2>
<p>Before we dive into how the effect works, let's establish some barebones markup:</p>

<pre class="figure__item codeblock"><code class="code code--html">&lt;div class="parallax"&gt;
  &lt;div class="parallax__layer parallax__layer--back"&gt;
    ...
  &lt;/div&gt;
  &lt;div class="parallax__layer parallax__layer--base"&gt;
    ...
  &lt;/div&gt;
&lt;/div&gt;</code></pre>

<p>And here are the basic style rules:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax {
  perspective: 1px;
  height: 100vh;
  overflow-x: hidden;
  overflow-y: auto;
}
.parallax__layer {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
}
.parallax__layer--base {
  transform: translateZ(0);
}
.parallax__layer--back {
  transform: translateZ(-1px);
}</code></pre>

<p>The <code>parallax</code> class is where the parallax magic happens. Defining the <code>height</code> and <code>perspective</code> style properties of an element will lock the perspective to its centre, creating a fixed origin 3D viewport. Setting <code>overflow-y: auto</code> will allow the content inside the element to scroll in the usual way, but now descendant elements will be rendered relative to the fixed perspective. This is the key to creating the parallax effect.</p>
<p>Next is the <code>parallax__layer</code> class. As the name suggests, it defines a layer of content to which the parallax effect will be applied; the element is pulled out of content flow and configured to fill the space of the container.</p>
<p>Finally we have the modifier classes <code>parallax__layer--base</code> and <code>parallax__layer--back</code>. These are used to determine the scrolling speed of a parallax element by translating it along the Z axis (moving it farther away, or closer to the viewport). For brevity I have only defined two layer speeds - we'll add more later.</p>
<p><a class="button" href="http://keithclark.co.uk/articles/pure-css-parallax-websites/demo1/">Try it</a></p>
<h2>Depth correction</h2>
<p>Since the parallax effect is created using 3D transforms, translating an element along the Z axis has a side effect - its effective size changes as we move it closer to or farther away from the viewport. To counter this we need to apply a <code>scale()</code> transform to the element so that it appears to be rendered at its original size:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax__layer--back {
  transform: translateZ(-1px) scale(2);
}</code></pre>

<p>The scale factor can be calculated with <code>1 + (translateZ * -1) / perspective</code>. For example, if our viewport <code>perspective</code> is set to <code>1px</code> and we translate an element <code>-2px</code> along the Z axis the correction scale factor would be 3:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax__layer--deep {
  transform: translateZ(-2px) scale(3);
}</code></pre>

<p><a class="button" href="http://keithclark.co.uk/articles/pure-css-parallax-websites/demo2/">Try it depth corrected</a></p>
<h2>Controlling layer speed</h2>
<p>Layer speed is controlled by a combination of the perspective and the Z translation values. Elements with negative Z values will scroll slower than those with a positive value. The further the value is from <code>0</code> the more pronounced the parallax effect (i.e. <code>translateZ(-10px)</code> will scroll slower than <code>translateZ(-1px)</code>).</p>
<h2>Parallax sections</h2>
<p>The previous examples demonstrated the basic techniques using very simple content but most parallax sites break the page into distinct sections where different effects can be applied. Here's how to do that.</p>
<p>Firstly, we need a <code>parallax__group</code> element to group our layers together:</p>

<pre class="figure__item codeblock"><code class="code code--html">&lt;div class="parallax"&gt;
  &lt;div class="parallax__group"&gt;
    &lt;div class="parallax__layer parallax__layer--back"&gt;
      ...
    &lt;/div&gt;
    &lt;div class="parallax__layer parallax__layer--base"&gt;
      ...
    &lt;/div&gt;
  &lt;/div&gt;
  &lt;div class="parallax__group"&gt;
    ...
  &lt;/div&gt;
&lt;/div&gt;</code></pre>

<p>Here's the CSS for the group element:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax__group {
  position: relative;
  height: 100vh;
  transform-style: preserve-3d;
}</code></pre>

<p>In this example, I want each group to fill the viewport so I've set <code>height: 100vh</code>, however arbitrary values can be set for each group if required. <code>transform-style: preserve-3d</code> prevents the browser flattening the <code>parallax__layer</code> elements and <code>position: relative</code> is used to allow the child <code>parallax__layer</code> elements to be positioned relative to the group element.</p>
<p>One important rule to keep in mind when grouping elements is, we cannot clip the content of a group. Setting <code>overflow: hidden</code> on a <code>parallax__group</code> will break the parallax effect. Unclipped content will result in descendant elements overflowing, so we need to be creative with the <code>z-index</code> values of the groups to ensure content is correctly revealed/hidden as the visitor scrolls through the document.</p>
<p>There are no hard and fast rules for dealing with layering as implementations will differ between designs. It's much easier to debug layering issues if you can see how the parallax effect works - you can do that by applying simple transform to the group elements:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax__group {
    transform: translate3d(700px, 0, -800px) rotateY(30deg);
}</code></pre>

<p>Have a look at the following example - note the <strong>debug</strong> option!</p>
<p><a class="button" href="http://keithclark.co.uk/articles/pure-css-parallax-websites/demo3/">Try it with groups</a></p>
<h2>Browser support</h2>
<ul class="bullet-list">
<li>Firefox, Safari, Opera and Chrome all support this effect.</li>
<li>Firefox works too but there is currently a minor issue with alignment.</li>
<li>IE doesn't support <code>preserve-3d</code> yet (it's coming) so the parallax effect won't work. That's ok though, you should still design your content to work without the parallax effect - You know, progressive enhancement and all that!</li>
</ul>
<hr/>
<h2>Update: 25 Feb 2015</h2>
<p>Since writing this article it's become apparent that webkit based browsers don't correctly calculate the effective width of an element once transformed into 3D space and scaled up. This bug allows users to scroll content horiziontally ignoring any constraits defined with <code>overflow</code> property. It is possible to work around for this bug by anchoring the <code>transform-origin</code> and <code>perspective-origin</code> to the right hand side of the viewport:</p>

<pre class="figure__item codeblock"><code class="code code--css">.parallax {
  perspective-origin-x: 100%;
}
.parallax__layer {
  transform-origin-x: 100%;
}</code></pre>

<p>Locking the perspective and transform origins in this way causes any content overflow to occur off the left side of the screen. Since it's not possible to scroll to a negative position the overflow issue is masked.</p>
<p><a class="button" href="http://keithclark.co.uk/articles/pure-css-parallax-websites/demo3-webkit-overflow-fix/">Try the webkit fix</a></p>
